@@ -17,8 +17,8 @@ from typing import (
     List,
     Any,
     Dict,
-    Callable,
     Hashable,
+    cast,
 )
 
 from typic.checks import (
@@ -27,9 +27,18 @@ from typic.checks import (
     isconstrained,
     issubclass,
     istypeddict,
+    isbuiltinsubtype,
+    isnamedtuple,
 )
 from typic.types import dsn, email, frozendict, path, secret, url
-from typic.util import origin, get_args, cached_signature, cached_type_hints, get_name
+from typic.util import (
+    origin,
+    get_args,
+    cached_signature,
+    cached_type_hints,
+    get_name,
+    TypeMap,
+)
 from .array import (
     Array,
     FrozenSetConstraints,
@@ -48,7 +57,7 @@ from .number import (
     IntContraints,
     FloatContraints,
     DecimalContraints,
-    Number,
+    NumberT,
 )
 from .text import BytesConstraints, StrConstraints
 
@@ -71,15 +80,17 @@ ConstraintsT = Union[
     TypeConstraints,
 ]
 
-_ARRAY_CONSTRAINTS_BY_TYPE: Mapping[
-    Type[Array],
-    Type[Union[SetContraints, ListContraints, TupleContraints, FrozenSetConstraints]],
-] = {
-    set: SetContraints,
-    list: ListContraints,
-    tuple: TupleContraints,
-    frozenset: FrozenSetConstraints,
-}
+_ARRAY_CONSTRAINTS_BY_TYPE = TypeMap(
+    {
+        set: SetContraints,
+        list: ListContraints,
+        tuple: TupleContraints,
+        frozenset: FrozenSetConstraints,
+    }
+)
+ArrayConstraintsT = Union[
+    SetContraints, ListContraints, TupleContraints, FrozenSetConstraints
+]
 
 
 def _resolve_args(*args, nullable: bool = False) -> Optional[ConstraintsT]:
@@ -105,9 +116,11 @@ def _resolve_args(*args, nullable: bool = False) -> Optional[ConstraintsT]:
 
 def _from_array_type(
     t: Type[Array], *, nullable: bool = False, name: str = None
-) -> ConstraintsT:
+) -> ArrayConstraintsT:
     args = get_args(t)
-    constr_class = _ARRAY_CONSTRAINTS_BY_TYPE[origin(t)]
+    constr_class = cast(
+        Type[ArrayConstraintsT], _ARRAY_CONSTRAINTS_BY_TYPE.get_by_parent(origin(t))
+    )
     # If we don't have args, then return a naive constraint
     if not args:
         return constr_class(nullable=nullable, name=name)
@@ -122,9 +135,8 @@ def _from_mapping_type(
     if isbuiltintype(t):
         return DictConstraints(nullable=nullable, name=name)
     base = getattr(t, "__origin__", t)
-    constr_class: Union[
-        Type[MappingConstraints], Type[DictConstraints]
-    ] = MappingConstraints
+    constr_class: Union[Type[MappingConstraints], Type[DictConstraints]]
+    constr_class = MappingConstraints
     if base is dict:
         constr_class = DictConstraints
     args = get_args(t)
@@ -137,23 +149,25 @@ def _from_mapping_type(
     )
 
 
-SimpleT = Union[Number, str, bytes]
+SimpleT = Union[NumberT, str, bytes]
 SimpleConstraintsT = Union[
     IntContraints, FloatContraints, DecimalContraints, StrConstraints, BytesConstraints
 ]
-_SIMPLE_CONSTRAINTS: Mapping[Type[SimpleT], Type[SimpleConstraintsT]] = {
-    IntContraints.type: IntContraints,
-    FloatContraints.type: FloatContraints,
-    DecimalContraints.type: DecimalContraints,
-    StrConstraints.type: StrConstraints,
-    BytesConstraints.type: BytesConstraints,
-}
+_SIMPLE_CONSTRAINTS = TypeMap(
+    {
+        IntContraints.type: IntContraints,
+        FloatContraints.type: FloatContraints,
+        DecimalContraints.type: DecimalContraints,
+        StrConstraints.type: StrConstraints,
+        BytesConstraints.type: BytesConstraints,
+    }
+)
 
 
 def _from_simple_type(
     t: Type[SimpleT], *, nullable: bool = False, name: str = None
 ) -> SimpleConstraintsT:
-    constr_class = _SIMPLE_CONSTRAINTS[t]
+    constr_class = cast(Type[SimpleConstraintsT], _SIMPLE_CONSTRAINTS.get_by_parent(t))
     return constr_class(nullable=nullable, name=name)
 
 
@@ -207,6 +221,8 @@ def _from_union(
 def _from_class(
     t: Type[VT], *, nullable: bool = False, name: str = None
 ) -> Union[ObjectConstraints, TypeConstraints, MappingConstraints]:
+    if not istypeddict(t) and not isnamedtuple(t) and isbuiltinsubtype(t):
+        return _from_strict_type(t, nullable=nullable, name=name)
     try:
         params: Dict[str, inspect.Parameter] = {**cached_signature(t).parameters}
         hints = cached_type_hints(t)
@@ -238,39 +254,42 @@ def _from_class(
     return cls(**kwargs)  # type: ignore
 
 
-_CONSTRAINT_BUILDER_HANDLERS: Mapping[Type[Any], Callable] = {
-    set: _from_array_type,
-    frozenset: _from_array_type,
-    list: _from_array_type,
-    tuple: _from_array_type,
-    dict: _from_mapping_type,
-    int: _from_simple_type,
-    float: _from_simple_type,
-    Decimal: _from_simple_type,
-    str: _from_simple_type,
-    bytes: _from_simple_type,
-    datetime.datetime: _from_strict_type,
-    datetime.date: _from_strict_type,
-    datetime.time: _from_strict_type,
-    url.NetworkAddress: _from_strict_type,
-    url.URL: _from_strict_type,
-    url.AbsoluteURL: _from_strict_type,
-    url.RelativeURL: _from_strict_type,
-    dsn.DSN: _from_strict_type,
-    pathlib.Path: _from_strict_type,
-    path.FilePath: _from_strict_type,
-    path.DirectoryPath: _from_strict_type,
-    path.PathType: _from_strict_type,
-    url.HostName: _from_strict_type,
-    email.Email: _from_strict_type,
-    secret.SecretStr: _from_strict_type,
-    secret.SecretBytes: _from_strict_type,
-    uuid.UUID: _from_strict_type,
-    re.Pattern: _from_strict_type,  # type: ignore
-    ipaddress.IPv4Address: _from_strict_type,
-    ipaddress.IPv6Address: _from_strict_type,
-    Union: _from_union,  # type: ignore
-}
+_CONSTRAINT_BUILDER_HANDLERS = TypeMap(
+    {
+        set: _from_array_type,
+        frozenset: _from_array_type,
+        list: _from_array_type,
+        tuple: _from_array_type,
+        dict: _from_mapping_type,
+        int: _from_simple_type,
+        float: _from_simple_type,
+        Decimal: _from_simple_type,
+        str: _from_simple_type,
+        bytes: _from_simple_type,
+        bool: _from_strict_type,
+        datetime.datetime: _from_strict_type,
+        datetime.date: _from_strict_type,
+        datetime.time: _from_strict_type,
+        url.NetworkAddress: _from_strict_type,
+        url.URL: _from_strict_type,
+        url.AbsoluteURL: _from_strict_type,
+        url.RelativeURL: _from_strict_type,
+        dsn.DSN: _from_strict_type,
+        pathlib.Path: _from_strict_type,
+        path.FilePath: _from_strict_type,
+        path.DirectoryPath: _from_strict_type,
+        path.PathType: _from_strict_type,
+        url.HostName: _from_strict_type,
+        email.Email: _from_strict_type,
+        secret.SecretStr: _from_strict_type,
+        secret.SecretBytes: _from_strict_type,
+        uuid.UUID: _from_strict_type,
+        re.Pattern: _from_strict_type,  # type: ignore
+        ipaddress.IPv4Address: _from_strict_type,
+        ipaddress.IPv6Address: _from_strict_type,
+        Union: _from_union,  # type: ignore
+    }
+)
 
 
 @functools.lru_cache(maxsize=None)
@@ -281,7 +300,9 @@ def get_constraints(
         return t.__constraints__  # type: ignore
     if issubclass(t, enum.Enum):
         return _from_enum_type(t, nullable=nullable, name=name)  # type: ignore
-
-    handler = _CONSTRAINT_BUILDER_HANDLERS.get(origin(t)) or _from_class
-    c = handler(t, nullable=nullable, name=name)
+    if isnamedtuple(t) or istypeddict(t):
+        handler = _from_class
+    else:
+        handler = _CONSTRAINT_BUILDER_HANDLERS.get_by_parent(origin(t), _from_class)  # type: ignore
+    c = handler(t, nullable=nullable, name=name)  # type: ignore
     return c
